@@ -1,5 +1,6 @@
 import inspect
 import importlib
+import struct
 from functools import cached_property
 from typing import TYPE_CHECKING, Optional, Any, Union
 
@@ -21,10 +22,7 @@ class MemoryProperty(property):
 
     @cached_property
     def pointer_format_string(self) -> str:
-        if self.memory_object.memobj_process.process_64_bit:
-            return "Q"
-        else:
-            return "I"
+        return self.memory_object.memobj_process.pointer_format_string
 
     def read_formatted_from_offset(self, format_string: str) -> tuple[Any] | Any:
         offset_address = self.memory_object.base_address + self.offset
@@ -34,12 +32,12 @@ class MemoryProperty(property):
         offset_address = self.memory_object.base_address + self.offset
         self.memory_object.memobj_process.write_formatted(offset_address, format_string, value)
 
-    def _get_prelude(self, memory_object):
-        self.memory_object = memory_object
+    def _get_prelude(self, preluder: "MemoryObject"):
+        self.memory_object = preluder
         return self.from_memory()
 
-    def _set_prelude(self, memory_object, value):
-        self.memory_object = memory_object
+    def _set_prelude(self, preluder: "MemoryObject", value):
+        self.memory_object = preluder
         self.to_memory(value)
 
     def from_memory(self) -> Any:
@@ -48,24 +46,34 @@ class MemoryProperty(property):
     def to_memory(self, value: Any):
         raise NotImplementedError()
 
+    def memory_size(self) -> int:
+        raise NotImplementedError()
 
-class ObjectPointer(MemoryProperty):
-    def __init__(
-            self,
-            offset: int | None,
-            object_type: Optional[Union[type["MemoryObject"], str]] = None,
-            *,
-            endianness: str = "little",
-    ):
+
+class Pointer(MemoryProperty):
+    """
+    name = Pointer(0x20, String(...))
+    other_type = Pointer(0x20, "NameOfType")
+    other_type2 = Pointer(0x20, OtherType())
+    position = Pointer(0x30, SimpleData(format_string="fff"))
+    """
+    def __init__(self, offset: int | None, pointed_type: Union[str, MemoryProperty, "MemoryObject"]):
         super().__init__(offset)
-        self.object_type = object_type
-        self._endianness = endianness
+        self.pointed_type = pointed_type
 
-    def _get_prelude(self, memory_object):
-        self.memory_object = memory_object
+    def from_memory(self) -> Any:
+        pointer = self.read_formatted_from_offset(self.pointer_format_string)
 
-        if isinstance(self.object_type, str):
-            module = memory_object.__module__
+        # this is to avoid circular import garbage
+        from memobj.object import MemoryObject
+
+        if isinstance(self.pointed_type, MemoryObject):
+            self.pointed_type._base_address = pointer
+            self.pointed_type.memobj_process = self.memory_object.memobj_process
+            return self.pointed_type
+
+        elif isinstance(self.pointed_type, str):
+            module = self.memory_object.__module__
 
             globals_ = None
             if __name__ == module:
@@ -77,40 +85,38 @@ class ObjectPointer(MemoryProperty):
             else:
                 module_import = importlib.import_module(module)
                 try:
-                    typed_object_type = getattr(module_import, module)
+                    typed_object_type = getattr(module_import, self.pointed_type)
                 except AttributeError:
-                    raise ValueError(f"{self.object_type} not found in scope of object")
+                    raise ValueError(f"{self.pointed_type} not found in scope of object")
 
             if globals_ is not None:
-                typed_object_type = globals_.get(self.object_type)
+                typed_object_type = globals_.get(self.pointed_type)
 
                 if typed_object_type is None:
-                    raise ValueError(f"{self.object_type} not found in scope of object")
+                    raise ValueError(f"{self.pointed_type} not found in scope of object")
 
-            self.object_type = typed_object_type
+            # noinspection PyUnboundLocalVariable
+            self.pointed_type = typed_object_type()
 
-        return self.from_memory()
+            self.pointed_type._base_address = pointer
+            self.pointed_type.memobj_process = self.memory_object.memobj_process
+            return self.pointed_type
 
-    @property
-    def endianness(self) -> str:
-        if self._endianness == "little":
-            return "<"
+        # MemoryProperty
+        elif isinstance(self.pointed_type, MemoryProperty):
+            # create a mock object at the address
+            self.pointed_type.memory_object = MemoryObject(
+                address=pointer,
+                process=self.memory_object.memobj_process,
+            )
+            self.pointed_type.offset = 0
+            return self.pointed_type.from_memory()
 
-        return ">"
+    def to_memory(self, value: Any):
+        pass
 
-    def from_memory(self) -> Any:
-        pointer = self.read_formatted_from_offset(self.endianness + self.pointer_format_string)
-
-        if pointer == 0:
-            return None
-
-        if self.object_type is None:
-            return type(self.memory_object)(pointer, self.memory_object.memobj_process)
-
-        return self.object_type(pointer, self.memory_object.memobj_process)
-
-    def to_memory(self, value: "MemoryObject"):
-        self.write_formatted_to_offset(self.pointer_format_string, value.base_address)
+    def memory_size(self) -> int:
+        return 8 if self.memory_object.memobj_process.process_64_bit else 4
 
 
 class NullTerminatedString(MemoryProperty):
@@ -162,67 +168,70 @@ class NullTerminatedString(MemoryProperty):
                 value,
             )
 
+    def memory_size(self) -> int:
+        if self.pointer:
+            return 8 if self.memory_object.memobj_process.process_64_bit else 4
+        else:
+            return self.max_size
 
-class SimpleDataProperty(MemoryProperty):
+
+class SimpleData(MemoryProperty):
     format_string: str = None
 
-    def __init__(self, offset: int | None, *, endianness: str = "little"):
+    def __init__(self, offset: int | None = None, format_string: str = None):
         super().__init__(offset)
-        self._endianness = endianness
-
-    @property
-    def endianness(self) -> str:
-        if self._endianness == "little":
-            return "<"
-
-        return ">"
+        if format_string is not None:
+            self.format_string = format_string
 
     def from_memory(self) -> Any:
-        return self.read_formatted_from_offset(self.endianness + self.format_string)
+        return self.read_formatted_from_offset(self.format_string)
 
     def to_memory(self, value: Any):
-        self.write_formatted_to_offset(self.endianness + self.format_string, value)
+        self.write_formatted_to_offset(self.format_string, value)
+
+    def memory_size(self) -> int:
+        return struct.calcsize(self.format_string)
 
 
-class Bool(SimpleDataProperty):
+class Bool(SimpleData):
     format_string = "?"
 
 
-class Float(SimpleDataProperty):
+class Float(SimpleData):
     format_string = "f"
 
 
-class Double(SimpleDataProperty):
+class Double(SimpleData):
     format_string = "d"
 
 
-class Signed1(SimpleDataProperty):
+class Signed1(SimpleData):
     format_string = "b"
 
 
-class Unsigned1(SimpleDataProperty):
+class Unsigned1(SimpleData):
     format_string = "B"
 
 
-class Signed2(SimpleDataProperty):
+class Signed2(SimpleData):
     format_string = "h"
 
 
-class Unsigned2(SimpleDataProperty):
+class Unsigned2(SimpleData):
     format_string = "H"
 
 
-class Signed4(SimpleDataProperty):
+class Signed4(SimpleData):
     format_string = "i"
 
 
-class Unsigned4(SimpleDataProperty):
+class Unsigned4(SimpleData):
     format_string = "I"
 
 
-class Signed8(SimpleDataProperty):
+class Signed8(SimpleData):
     format_string = "q"
 
 
-class Unsigned8(SimpleDataProperty):
+class Unsigned8(SimpleData):
     format_string = "Q"
